package dev.fyloz.trial.colorrecipesexplorer.core.services.model;

import dev.fyloz.trial.colorrecipesexplorer.core.io.file.FileHandler;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Material;
import dev.fyloz.trial.colorrecipesexplorer.core.model.MaterialType;
import dev.fyloz.trial.colorrecipesexplorer.core.services.GenericService;
import dev.fyloz.trial.colorrecipesexplorer.dao.MaterialDao;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.web.multipart.MultipartFile;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Comparator;
import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;

@Service
public class MaterialService extends GenericService<Material, MaterialDao> {

    private MixQuantityService mixQuantityService;

    @Autowired
    public MaterialService(MaterialDao materialDao, MixQuantityService mixQuantityService) {
        super(materialDao);
        this.mixQuantityService = mixQuantityService;
    }

    public Optional<Material> getByName(String name) {
        return dao.findByName(name);
    }

    public List<Material> getAllByMaterialType(MaterialType materialType) {
        if (materialType == null) return new ArrayList<>();

        return dao.findAllByMaterialType(materialType);
    }

    public List<Material> getAllOrdered() {
        return getAll().stream()
                .sorted(Comparator.comparing(Material::getName))
                .collect(Collectors.toList());
    }

    /**
     * Vérifie si un produit est lié à un ou plusieurs mélanges.
     *
     * @param material Le produit à vérifier.
     * @return Si le produit est lié à d'autres mélanges.
     */
    public boolean isLinkedToMixes(Material material) {
        return mixQuantityService.existsByMaterial(material);
    }

    public boolean deleteIfNotLinked(Material material) {
        if (!isLinkedToMixes(material)) {
            return delete(material);
        }

        return false;
    }

    @Override
    public boolean exists(Material material) {
        return material != null && (super.exists(material) || dao.existsByName(material.getName()));
    }

    @Override
    public boolean isValidForUpdate(Material material) {
        if (material == null) return false;

        Optional<Material> materialByCode = dao.findByName(material.getName());
        return super.isValidForUpdate(material) && (materialByCode.isEmpty() || material.getId().equals(materialByCode.get().getId()));
    }

    @Deprecated(since = "1.2.0")
    public List<Material> getAllBySearchString(String searchString) {
        return dao.findAllByNameContainingIgnoreCase(searchString).stream().filter(m -> !m.isMixType()).collect(Collectors.toList());
    }

    /**
     * Crée un FileHandler pour le produit passé en paramètre.
     *
     * @param material Le produit dont on veut créer un FileHandler
     * @return Le FileHandler correspondant au produit.
     */
    private FileHandler getFileHandlerForMaterial(Material material) {
        String filename = String.format("%s_%s", material.getId(), material.getName());
        return new FileHandler(filename, FileHandler.FileContext.SIMDUT, FileHandler.FileExtension.PDF);
    }

    /**
     * Crée le fichier SIMDUT sur le disque et y transfert le contenu du MultipartFile passé en paramètre.
     * <p>
     * Cette méthode retournera true si le simdut est null ou si le transfert du fichier vers le disque a fonctionné.
     * Cette méthode retournera false si la création du fichier échoue ou si une erreur survient lors du transfert.
     *
     * @param simdut   Le contenu du fichier SIMDUT
     * @param material Le produit du SIMDUT
     * @return Si le fichier SIMDUT à bien été créé ou non.
     */
    public boolean addSimdut(MultipartFile simdut, Material material) {
        if (simdut.getSize() <= 0) return false;

        FileHandler fileHandler = getFileHandlerForMaterial(material);

        if (!fileHandler.createFile()) {
            return false;
        }

        try {
            simdut.transferTo(new File(fileHandler.getFile().getAbsolutePath()));
            return true;
        } catch (IOException e) {
            logger.error("Erreur lors du transfert d'un fichier SIMDUT vers le disque", e);
            return false;
        }
    }

    /**
     * Supprime le fichier SIMDUT pour un produit.
     * <p>
     * Cette méthode retournera true si le fichier SIMDUT du produit n'existe pas ou si la suppression est faîtes.
     *
     * @param material Le produit dont on veut supprimer le fichier SIMDUT
     * @return Si la suppression du fichier SIMDUT s'est effectuée.
     */
    public boolean removeSimdut(Material material) {
        FileHandler fileHandler = getFileHandlerForMaterial(material);

        if (fileHandler.getFile().exists()) {
            return fileHandler.deleteFile();
        }

        return true;
    }
}
