package dev.fyloz.trial.colorrecipesexplorer.core.services.model;

import dev.fyloz.trial.colorrecipesexplorer.core.io.file.FileHandler;
import dev.fyloz.trial.colorrecipesexplorer.core.io.file.ImageHandler;
import dev.fyloz.trial.colorrecipesexplorer.core.model.*;
import dev.fyloz.trial.colorrecipesexplorer.core.services.GenericService;
import dev.fyloz.trial.colorrecipesexplorer.dao.RecipeDao;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.util.MultiValueMap;

import java.io.File;
import java.util.*;
import java.util.stream.Collectors;

@Service
public class RecipeService extends GenericService<Recipe, RecipeDao> {

    private CompanyService companyService;
    private StepService stepService;

    @Autowired
    public RecipeService(RecipeDao recipeDao, CompanyService companyService, StepService stepService) {
        super(recipeDao);
        this.companyService = companyService;
        this.stepService = stepService;
    }

    /**
     * Récupère la liste des recettes associées à une compagnie.
     *
     * @param company La compagnie.
     * @return La liste des recettes associées à cette compagnie.
     */
    public List<Recipe> getByCompany(Company company) {
        return dao.findAllByCompany(company);
    }

    /**
     * Récupère une liste triée des mélanges pour une recette.
     *
     * @param recipe La recette dont on veut récupérer les mélanges
     * @return Une liste triée des mélanges.
     */
    public List<Mix> getSortedMixes(Recipe recipe) {
        List<Mix> mixes = recipe.getMixes();
        mixes.sort(Comparator.comparing(Mix::getId));
        return new ArrayList<>(mixes);  // Convertit le PersistentBag en ArrayList
    }

    /**
     * Récupère toutes les recettes dans la base de données et les classes par compagnie.
     *
     * @return Un Map contenant les recettes classées par compagnie.
     */
    public Map<Company, List<Recipe>> getRecipesByCompany() {
        return mappedByCompany(getAll());
    }

    public Optional<Recipe> updateRecipe(Recipe newRecipe, Recipe storedRecipe, MultiValueMap<String, Object> form) {
        storedRecipe.setName(newRecipe.getName());
        storedRecipe.setCompany(newRecipe.getCompany());
        storedRecipe.setDescription(newRecipe.getDescription());
        storedRecipe.setSample(newRecipe.getSample());
        storedRecipe.setApprobationDate(newRecipe.getApprobationDate());
        storedRecipe.setRemark(newRecipe.getRemark());
        storedRecipe.setNote(newRecipe.getNote());

        Optional<Recipe> optionalRecipe = convertAndCreateSteps(storedRecipe, form);
        if (optionalRecipe.isEmpty()) {
            return Optional.empty();
        }

        return update(optionalRecipe.get());
    }

    @Transactional
    public boolean deleteRecipe(Recipe recipe) {
        if (!delete(recipe)) return false;
        return removeAllFiles(recipe);
    }

    /**
     * Récupère le nom des images liées à une recette.
     *
     * @param recipe La recette dont on veut récupérer les images
     * @return Une liste contenant le nom des images liées à la recette.
     */
    public List<String> getImageFiles(Recipe recipe) {
        Long recipeId = recipe.getId();
        String name = recipe.getName();
        String fileName = String.format("%s_%s", recipeId, name);

        File imageLocation = new File(ImageHandler.IMAGES_LOCATION);
        File[] result = imageLocation.listFiles((d, n) -> n.startsWith(fileName) && n.endsWith("jpeg"));

        if (result == null) return new ArrayList<>();

        return Arrays.stream(result).map(File::getName).collect(Collectors.toList());
    }

    /**
     * Récupère les types de mélanges associés à la recette.
     *
     * @param recipe La recette dont on veut récupérer les types de mélange
     * @return Une liste contenant les types de mélanges.
     */
    public List<MixType> getAssociatedMixesTypes(Recipe recipe) {
        return recipe
                .getMixes()
                .stream()
                .map(Mix::getMixType)
                .collect(Collectors.toList());
    }

    /**
     * Vérifie si une recette contient un mélange ayant le type de mélange spécifié.
     *
     * @param recipe  La recette
     * @param mixType Le type de mélange
     * @return Si la recette contient le type de mélange
     */
    public boolean hasMixType(Recipe recipe, MixType mixType) {
        return getAssociatedMixesTypes(recipe)
                .contains(mixType);
    }

    @Deprecated(since = "1.2.0")
    public Map<Company, List<Recipe>> getRecipesForSearchString(String searchString) {
        List<Recipe> recipes = dao.findAllByRecipeDescriptionContainsOrRecipeCodeContains(searchString.toUpperCase());

        return mappedByCompany(recipes);
    }

    private Map<Company, List<Recipe>> mappedByCompany(List<Recipe> recipes) {
        List<Company> companies = companyService.getAll();
        Map<Company, List<Recipe>> mappedRecipes = new HashMap<>();

        for (Company c : companies) {
            List<Recipe> recipesForCompany = recipes.stream().filter(r -> r.getCompany().equals(c)).collect(Collectors.toList());

            if (recipesForCompany.size() > 0) {
                mappedRecipes.put(c, recipesForCompany);
            }
        }

        return mappedRecipes;
    }

    /**
     * Crée et assigne des étapes à une recette, en se basant sur l'entrée de utilisateur.
     *
     * @param recipe La recette à assigner les étapes
     * @param input  L'entrée de l'utilisateur, contenant les étapes (key = "step_*")
     * @return La recette avec les étapes.
     */
    @Transactional
    public Optional<Recipe> convertAndCreateSteps(Recipe recipe, MultiValueMap<String, Object> input) {
        if (recipe == null || input == null) return Optional.empty();

        // Convertit les étapes en RecipeSteps
        List<RecipeStep> steps = new ArrayList<>();
        if (input.containsKey("step")) {
            Object field = input.get("step");

            if (field instanceof LinkedList && ((LinkedList) field).getFirst() instanceof String) {
                for (String step : (LinkedList<String>) field) {
                    steps.add(new RecipeStep(recipe, step));
                }
            }
        }

        return setSteps(recipe, steps);
    }

    private Optional<Recipe> setSteps(Recipe recipe, List<RecipeStep> steps) {
        if (!stepService.deleteAll(recipe.getRecipeSteps())) return Optional.empty();

        recipe.setRecipeSteps(null);
        update(recipe);

        recipe.setRecipeSteps(steps);
        return Optional.of(recipe);
    }

    private boolean removeAllFiles(Recipe recipe) {
        FileHandler fileHandler;
        for (String image : getImageFiles(recipe)) {
            fileHandler = new FileHandler(image.replace(".jpeg", ""), FileHandler.FileContext.IMAGE, FileHandler.FileExtension.JPEG);

            if (!fileHandler.deleteFile()) return false;
        }

        return true;
    }
}
