package dev.fyloz.trial.colorrecipesexplorer.web.controller.editors;

import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseCode;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseDataType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Material;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MaterialService;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MaterialTypeService;
import dev.fyloz.trial.colorrecipesexplorer.core.utils.ControllerUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.MediaType;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

import java.util.Optional;
import java.util.stream.Collectors;

import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.*;

@Controller
public class MaterialEditorController {

    private MaterialService materialService;
    private MaterialTypeService materialTypeService;

    @Autowired
    public MaterialEditorController(MaterialService materialService, MaterialTypeService materialTypeService) {
        this.materialService = materialService;
        this.materialTypeService = materialTypeService;
    }

    /**
     * Affiche la page de tous les produits.
     * <p>
     * Modèle de la page:
     * - materials: La liste de tous les produits
     *
     * @param model Le Model injecté par Thymeleaf
     * @return La page à afficher.
     */
    @GetMapping(EDITOR_MATERIAL)
    public ModelAndView listMaterials(ModelAndView model) {
        return new ModelResponseBuilder(model)
                .withView(EDITOR_MATERIAL)
                .addResponseData(ResponseDataType.MATERIALS, materialService.getAll().stream().filter(m -> !m.isMixType()).collect(Collectors.toList()))
                .build();
    }

    @GetMapping(EDITOR_MATERIAL_SPECIFIC)
    public ModelAndView showEditPage(ModelAndView model, @PathVariable Long id, Material material) {
        ModelResponseBuilder responseBuilder = new ModelResponseBuilder(model)
                .withView(EDITOR_MATERIAL_EDITOR);

        if (material.getName() == null) {
            Optional<Material> optionalMaterial = materialService.getById(id);

            if (optionalMaterial.isEmpty()) {
                return listMaterials(
                        responseBuilder
                                .addResponseCode(ResponseCode.MATERIAL_NOT_FOUND, id)
                                .build()
                );
            }

            material = optionalMaterial.get();
        }

        return responseBuilder
                .addResponseData(ResponseDataType.MATERIAL, material)
                .addResponseData(ResponseDataType.MATERIAL_TYPES, materialTypeService.getAll())
                .build();
    }

    /**
     * Permet à l'utilisateur de modifier un produit.
     * <p>
     * La modification échouera si:
     * - L'utilisateur n'est pas autorisé à exécuter cette action
     * - Il n'y a aucun produit avec l'identifiant passé en paramètre
     * - Une erreur est survenue lors de la mise à jour dans la base de données
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * - materialCode: Contient le code du produit mit à jour
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param material Le produit à mettre à jour
     * @return La page à afficher.
     */
    @PostMapping(value = EDITOR_MATERIAL, consumes = MediaType.APPLICATION_FORM_URLENCODED_VALUE)
    public ModelAndView saveEditedMaterial(Material material) {
        ModelResponseBuilder responseBuilder = new ModelResponseBuilder("");
        Long id = material.getId();

        if (!materialService.existsById(id)) {
            responseBuilder.addResponseCode(ResponseCode.MATERIAL_NOT_FOUND, id);
        } else if (!materialService.isValidForUpdate(material)) {
            responseBuilder.addResponseCode(ResponseCode.MATERIAL_ALREADY_EXIST, material.getName());

            return showEditPage(
                    responseBuilder
                            .addResponseCode(ResponseCode.MATERIAL_ALREADY_EXIST, material.getName())
                            .build(),
                    id, material);
        } else {
            Optional<Material> updatedMaterial = materialService.update(material);
            if (updatedMaterial.isPresent()) {
                responseBuilder.addResponseCode(ResponseCode.SUCCESS_SAVING_MATERIAL, updatedMaterial.get().getName());
            } else {
                return showEditPage(
                        responseBuilder
                                .addResponseCode(ResponseCode.ERROR_SAVING)
                                .build(),
                        id, null);
            }
        }

        return listMaterials(responseBuilder.build());
    }

    /**
     * Affiche la page d'upload d'un fichier SIMDUT pour un produit
     * Cette méthode requiert l'identifiant du produit dans l'URL
     * <p>
     * Modèle de la page:
     * - id: Contient l'identifiant du produit
     *
     * @param model      Le Model injecté par Thymeleaf
     * @param id L'identifiant du produit à modifier le SIMDUT
     * @return La page à afficher.
     */
    @GetMapping(value = EDIT_MATERIAL_SIMDUT_SPECIFIC)
    public ModelAndView chooseSIMDUTFile(ModelAndView model, @PathVariable Long id) {
        return new ModelResponseBuilder(model)
                .withView(EDIT_MATERIAL_SIMDUT)
                .addResponseData(ResponseDataType.MATERIAL_ID, id)
                .build();
    }

    /**
     * Sauvegarde le fichier SIMDUT d'un produit.
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param id L'identifiant du produit
     * @param simdut     Le fichier SIMDUT uploadé
     * @return La page à afficher.
     */
    @PostMapping(value = EDIT_MATERIAL_SIMDUT, consumes = MediaType.MULTIPART_FORM_DATA_VALUE)
    public ModelAndView saveSIMDUT(Long id, MultipartFile simdut) {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder(ControllerUtils.redirect("material/editor/" + id));

        Optional<Material> optionalMaterial = materialService.getById(id);
        if (optionalMaterial.isEmpty()) {
            return chooseSIMDUTFile(modelResponseBuilder
                            .addResponseCode(ResponseCode.MATERIAL_NOT_FOUND, id)
                            .build(),
                    id);
        }

        Material material = optionalMaterial.get();
        if (!materialService.removeSimdut(material) || !materialService.addSimdut(simdut, material)) {
            modelResponseBuilder.addResponseCode(ResponseCode.ERROR_SAVING_SIMDUT);

            return chooseSIMDUTFile(modelResponseBuilder.build(), id);
        }

        return modelResponseBuilder.build();
    }
}
