package dev.fyloz.trial.colorrecipesexplorer.web.controller.files;

import dev.fyloz.trial.colorrecipesexplorer.ColorRecipesExplorerApplication;
import dev.fyloz.trial.colorrecipesexplorer.core.io.file.ImageHandler;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.JSONResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ModelResponseBuilder;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseCode;
import dev.fyloz.trial.colorrecipesexplorer.core.io.response.ResponseDataType;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Recipe;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.RecipeService;
import dev.fyloz.trial.colorrecipesexplorer.core.utils.ControllerUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpHeaders;
import org.springframework.http.HttpStatus;
import org.springframework.http.MediaType;
import org.springframework.http.ResponseEntity;
import org.springframework.stereotype.Controller;
import org.springframework.web.bind.annotation.*;
import org.springframework.web.multipart.MultipartFile;
import org.springframework.web.servlet.ModelAndView;

import javax.imageio.ImageIO;
import java.io.File;
import java.io.IOException;
import java.util.Map;
import java.util.Optional;

import static dev.fyloz.trial.colorrecipesexplorer.web.WebsitePaths.*;

@Controller
public class ImageFilesController {

    private RecipeService recipeService;

    @Autowired
    public ImageFilesController(RecipeService recipeService) {
        this.recipeService = recipeService;
    }

    /**
     * Récupère l'image voulue sous forme JPEG.
     *
     * @param image Le nom de l'image
     * @return L'image en JPEG.
     */
    @GetMapping(IMAGES_FILES)
    public ResponseEntity<byte[]> getImage(@PathVariable String image) {
        ImageHandler imageHandler = new ImageHandler(image, recipeService);

        byte[] content = imageHandler.readFile();
        if (content == null) return ResponseEntity.notFound().build();

        HttpHeaders headers = new HttpHeaders();
        headers.setContentType(MediaType.IMAGE_JPEG);
        return new ResponseEntity<>(content, headers, HttpStatus.OK);
    }

    /**
     * Affiche la page pour ajouter une image pour une recette.
     * Cette méthode requiert l'identifiant de la recette dans l'URL.
     * <p>
     * Modèle de la page:
     * - id: Contient l'identifiant de la recette
     *
     * @param model    Le Model injecté par Thymeleaf
     * @param id L'identifiant de la recette
     * @return La page à afficher.
     */
    @GetMapping(ADD_IMAGE_SPECIFIC)
    public ModelAndView getPage(ModelAndView model, @PathVariable Long id) {
        return new ModelResponseBuilder(model)
                .withView(ADD_IMAGE)
                .addResponseData(ResponseDataType.RECIPE_ID, id)
                .build();
    }

    /**
     * Permet à l'utilisateur d'uploader une image.
     * <p>
     * L'upload échouera si:
     * - L'utilisateur n'est pas autorisé à exécuter cette action.
     * - La recette n'existe pas
     * - Une erreur est survenue lors de la création du fichier de l'image
     * - Une erreur est survenue lors du transfert de l'image vers le fichier
     * <p>
     * Modèle de la page:
     * - error: Contient le message d'erreur, s'il y a lieu
     * - recipeCode: Contient la couleur de la recette
     * - id: Contient l'identifiant de la recette
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param id L'identifiant de la recette
     * @param image    L'image uploadée
     * @return La page à afficher.
     */
    @PostMapping(ADD_IMAGE)
    public ModelAndView addImage(Long id, MultipartFile image) throws IOException {
        ModelResponseBuilder modelResponseBuilder = new ModelResponseBuilder(ControllerUtils.redirect(EDITOR_RECIPE_SPECIFIC.replace("{id}", String.valueOf(id))));

        // Vérifie que le fichier est bien une image
        if (ImageIO.read(image.getInputStream()) == null) {
            return getPage(modelResponseBuilder
                    .addResponseCode(ResponseCode.FILE_NOT_IMAGE)
                    .build(), id);
        }

        Optional<Recipe> optionalRecipe = recipeService.getById(id);
        if (optionalRecipe.isEmpty()) {
            return getPage(modelResponseBuilder
                    .addResponseCode(ResponseCode.RECIPE_NOT_FOUND, id)
                    .build(), id);
        }

        Recipe recipe = optionalRecipe.get();
        ImageHandler imageHandler = new ImageHandler(recipe, recipeService);
        if (!imageHandler.createFile()) {
            return getPage(modelResponseBuilder
                    .addResponseCode(ResponseCode.ERROR_SAVING_IMAGE)
                    .build(), id);
        }

        modelResponseBuilder
                .addResponseData(ResponseDataType.RECIPE_CODE, recipe.getName())
                .addResponseData(ResponseDataType.RECIPE_ID, recipe.getId());

        try {
            // Si je n'utilise pas le path, il cherche un fichier dans les tmp ?
            image.transferTo(new File(imageHandler.getFile().getAbsolutePath()));

            return modelResponseBuilder.build();
        } catch (IOException e) {
            ColorRecipesExplorerApplication.LOGGER.error("Erreur inconnue lors de la création d'une image", e);
            modelResponseBuilder.addResponseCode(ResponseCode.ERROR_SAVING_IMAGE);

            return getPage(modelResponseBuilder.build(), id);
        }
    }

    /**
     * Permet à l'utilisateur de supprimer une image.
     * <p>
     * La suppression échouera si:
     * - L'utilisateur n'est pas autorisé à exécuter cette action
     * - Une erreur est survenue lors de la suppression du fichier
     * <p>
     * Réponse de la méthode:
     * - error: Contient le message d'erreur, s'il y a lieu
     * <p>
     * REQUIERT UNE AUTORISATION
     *
     * @param form Le formulaire contenant les informations sur l'opération
     * @return La réponse sous forme de Map (pour y accéder en Json)
     */
    @PostMapping(value = DELETE_IMAGE, consumes = MediaType.APPLICATION_JSON_VALUE, produces = MediaType.APPLICATION_JSON_VALUE)
    @ResponseBody
    public Map<String, Object> deleteImage(@RequestBody Map<String, String> form) {
        JSONResponseBuilder responseBuilder = new JSONResponseBuilder();

        ImageHandler imageHandler = new ImageHandler(form.get("image"), recipeService);
        if (!imageHandler.deleteFile()) {
            responseBuilder.addResponseCode(ResponseCode.ERROR_SAVING_IMAGE);
        }

        return responseBuilder.build();
    }
}
