package dev.fyloz.trial.colorrecipesexplorer.core.services;

import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MaterialService;
import dev.fyloz.trial.colorrecipesexplorer.core.model.Material;
import dev.fyloz.trial.colorrecipesexplorer.core.model.MaterialType;
import dev.fyloz.trial.colorrecipesexplorer.core.services.model.MaterialTypeService;
import dev.fyloz.trial.colorrecipesexplorer.dao.MaterialTypeDao;
import org.junit.Before;
import org.junit.Test;
import org.mockito.InjectMocks;
import org.mockito.Mock;
import org.mockito.MockitoAnnotations;
import org.mockito.Spy;

import java.util.ArrayList;
import java.util.List;
import java.util.Optional;

import static org.junit.Assert.*;
import static org.mockito.ArgumentMatchers.any;
import static org.mockito.Mockito.doReturn;

public class MaterialTypeServiceTests {

    @Mock
    private MaterialService materialService;

    @Mock
    private MaterialTypeDao materialTypeDao;

    @Spy
    @InjectMocks
    private MaterialTypeService materialTypeService;

    private MaterialType testMaterialType;
    private Material testMaterial;

    @Before
    public void init() {
        MockitoAnnotations.initMocks(this);

        testMaterialType = new MaterialType();
        testMaterialType.setMaterialTypeID(0);
        testMaterialType.setMaterialTypeName("Test Type");

        testMaterial = new Material();
        testMaterial.setMaterialID(0);
        testMaterial.setName("Test Material");
    }

    @Test
    public void isLinkedToMaterialsTestShouldBeTrue() {
        // Arrange
        List<Material> testMaterialList = new ArrayList<>();
        testMaterialList.add(testMaterial);

        doReturn(testMaterialList).when(materialService).getAllByMaterialType(testMaterialType);

        // Act
        boolean actual = materialTypeService.isLinkedToMaterials(testMaterialType);

        // Assert
        assertTrue(actual);
    }

    @Test
    public void isLinkedToMaterialsTestShouldBeFalse() {
        doReturn(new ArrayList<>()).when(materialService).getAllByMaterialType(testMaterialType);

        boolean actual = materialTypeService.isLinkedToMaterials(testMaterialType);
        assertFalse(actual);
    }

    @Test
    public void isLinkedToMaterialsTestNull() {
        doReturn(new ArrayList<>()).when(materialService).getAllByMaterialType(any());

        boolean actual = materialTypeService.isLinkedToMaterials(testMaterialType);
        assertFalse(actual);
    }

    @Test
    public void deleteIfNotLinkedTestShouldBeTrue() {
        doReturn(false).when(materialTypeService).isLinkedToMaterials(testMaterialType);
        doReturn(true).when(materialTypeService).delete(testMaterialType);

        boolean actual = materialTypeService.deleteIfNotLinked(testMaterialType);
        assertTrue(actual);
    }

    @Test
    public void deleteIfNotLinkedTestShouldBeFalse() {
        doReturn(true).when(materialTypeService).isLinkedToMaterials(testMaterialType);
        doReturn(false).when(materialTypeService).delete(testMaterialType);

        boolean actual = materialTypeService.deleteIfNotLinked(testMaterialType);
        assertFalse(actual);
    }

    @Test
    public void deleteIfNotLinkedTestIsLinked() {
        doReturn(true).when(materialTypeService).isLinkedToMaterials(testMaterialType);
        doReturn(true).when(materialTypeService).delete(testMaterialType);

        boolean actual = materialTypeService.deleteIfNotLinked(testMaterialType);
        assertFalse(actual);
    }

    @Test
    public void deleteIfNotLinkedTestNull() {
        doReturn(true).when(materialTypeService).isLinkedToMaterials(testMaterialType);
        doReturn(false).when(materialTypeService).delete(testMaterialType);

        boolean actual = materialTypeService.deleteIfNotLinked(null);
        assertFalse(actual);
    }

    @Test
    public void isValidForCreationTestShouldBeTrue() {
        doReturn(false).when(materialTypeService).existsByName(testMaterialType.getName());

        boolean actual = materialTypeService.isValidForCreation(testMaterialType);
        assertTrue(actual);
    }

    @Test
    public void isValidForCreationTestShouldBeFalse() {
        doReturn(true).when(materialTypeService).existsByName(testMaterialType.getName());

        boolean actual = materialTypeService.isValidForCreation(testMaterialType);
        assertFalse(actual);
    }

    @Test
    public void isValidForCreationTestNull() {
        doReturn(false).when(materialTypeService).existsByName(testMaterialType.getName());

        boolean actual = materialTypeService.isValidForCreation(null);
        assertFalse(actual);
    }

    @Test
    public void isValidForUpdateTestShouldBeTrue() {
        doReturn(true).when(materialTypeService).exists(testMaterialType);
        doReturn(true).when(materialTypeService).isValidForUpdateName(testMaterialType);
        doReturn(true).when(materialTypeService).isValidForUpdatePrefix(testMaterialType);

        boolean actual = materialTypeService.isValidForUpdate(testMaterialType);
        assertTrue(actual);
    }

    @Test
    public void isValidForUpdateTestShouldBeFalse() {
        doReturn(false).when(materialTypeService).exists(testMaterialType);
        doReturn(false).when(materialTypeService).isValidForUpdateName(testMaterialType);
        doReturn(false).when(materialTypeService).isValidForUpdatePrefix(testMaterialType);

        boolean actual = materialTypeService.isValidForUpdate(testMaterialType);
        assertFalse(actual);
    }

    @Test
    public void isValidForUpdateTestNull() {
        doReturn(false).when(materialTypeService).exists(any());
        doReturn(false).when(materialTypeService).isValidForUpdateName(any());
        doReturn(false).when(materialTypeService).isValidForUpdatePrefix(any());

        boolean actual = materialTypeService.isValidForUpdate(null);
        assertFalse(actual);
    }

    @Test
    public void getByNameTest() {
        doReturn(Optional.of(testMaterialType)).when(materialTypeDao).findByMaterialTypeName(testMaterialType.getName());

        Optional<MaterialType> actual = materialTypeService.getByName(testMaterialType.getName());
        assertTrue(actual.isPresent());
        assertEquals(testMaterialType.getName(), actual.get().getName());
    }

    @Test
    public void getByNameTestShouldBeEmpty() {
        doReturn(Optional.empty()).when(materialTypeDao).findByMaterialTypeName(testMaterialType.getName());

        Optional<MaterialType> actual = materialTypeService.getByName(testMaterialType.getName());
        assertFalse(actual.isPresent());
    }

    @Test
    public void getByNameTestNull() {
        doReturn(Optional.empty()).when(materialTypeDao).findByMaterialTypeName(testMaterialType.getName());

        Optional<MaterialType> actual = materialTypeService.getByName(null);
        assertFalse(actual.isPresent());
    }

    @Test
    public void getDefaultMaterialTypeTest() {
        doReturn(Optional.of(testMaterialType)).when(materialTypeService).getByName(MaterialType.DEFAULT_MATERIAL_TYPE_NAME);

        Optional<MaterialType> actual = materialTypeService.getDefaultMaterialType();
        assertTrue(actual.isPresent());
    }

    // Test non-fonctionnel
//    @Test
//    public void getDefaultMaterialTypeGetByNameFail() {
//        doReturn(Optional.empty()).when(materialTypeService).getByName(any());
//
//        Optional<MaterialType> actual = materialTypeService.getDefaultMaterialType();
//        assertTrue(actual.isPresent());
//    }

    @Test
    public void existsByNameTestShouldBeTrue() {
        doReturn(Optional.of(testMaterialType)).when(materialTypeService).getByName(testMaterialType.getName());

        boolean actual = materialTypeService.existsByName(testMaterialType.getName());
        assertTrue(actual);
    }

    @Test
    public void existsByNameTestShouldBeFalse() {
        doReturn(Optional.empty()).when(materialTypeService).getByName(testMaterialType.getName());

        boolean actual = materialTypeService.existsByName(testMaterialType.getName());
        assertFalse(actual);
    }

    @Test
    public void existsByNameTestNull() {
        doReturn(Optional.empty()).when(materialTypeService).getByName(any());

        boolean actual = materialTypeService.existsByName(null);
        assertFalse(actual);
    }

}
